@@ -1,4 +1,4 @@
-#!/usr/pkg/bin/python2.7
+#!/21Q1/bin/python2.7
 # -*- mode: python -*-
 
 # Re test suite and benchmark suite v1.5
@@ -663,7 +663,7 @@ xyzabc
 
 try:
     u = eval("u'\N{LATIN CAPITAL LETTER A WITH DIAERESIS}'")
-except SyntaxError:
+except (SyntaxError, ValueError):
     pass
 else:
     tests.extend([
